#ifndef GENEPAIR_SHORTEST_PATH_DATABASE_H
#define GENEPAIR_SHORTEST_PATH_DATABASE_H

#include <vector>
#include <string>
#include <cassert>
#include <tcutil.h>
#include <tchdb.h>
#include "maxlink_types.h"
#include "hdb_tokyo_cabinet_database.h"

class GenePairShortestPathDatabase : public HdbTokyoCabinetDatabase {
 public:

   GenePairShortestPathDatabase() {};
  ~GenePairShortestPathDatabase() {};
  void storeShortestPath(maxlink_types::GenePair gene_pair, maxlink_types::ShortestPathId shortest_path_id) {
    assert(init_run_);
    assert(hdb_);
    if (gene_pair.first < gene_pair.second) {
      // The gene pairs are always stored with the biggest gene first in the pair
      maxlink_types::GeneId tmp_gene_id = gene_pair.first;
      gene_pair.first = gene_pair.second;
      gene_pair.second = tmp_gene_id;
    }

    if (!tchdbput(hdb_, &gene_pair, sizeof(gene_pair), &shortest_path_id, sizeof(shortest_path_id))) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error in tchdbputcat: %s\n", tchdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
  }
  std::string endingFilename() {
    return std::string("GenePairShortestPathDatabase.hdb");
  };

  bool shortestPath(maxlink_types::GenePair gene_pair, maxlink_types::ShortestPathId *shortest_path_id) {
     // For the moment being, this function only returns  one shortest path. 
     //
     // We might try to find all shortest paths betweeen two genes
     // http://lists.boost.org/boost-users/2010/11/63878.php
     // 
     // The number of shortest paths could possibly be very large, so it might lead to a scaling problem (storage, compute time).

    assert(init_run_);
    assert(hdb_);

    if (gene_pair.first < gene_pair.second) {
      // The gene pairs are always stored with the biggest gene first in the pair
      maxlink_types::GeneId tmp_gene_id = gene_pair.first;
      gene_pair.first = gene_pair.second;
      gene_pair.second = tmp_gene_id;
    }
    int value_size = 0;
    maxlink_types::ShortestPathId *result_ptr = static_cast<maxlink_types::ShortestPathId *>(tchdbget(hdb_, &gene_pair, sizeof(gene_pair), &value_size));
    if (!result_ptr) {
      return false;
    } 
    assert(value_size == sizeof(maxlink_types::ShortestPathId));
    *shortest_path_id = *result_ptr;
    free(result_ptr);
    return true;
  }
};

#endif
