#ifndef HDB_TOKYO_CABINET_DATABASE_H
#define HDB_TOKYO_CABINET_DATABASE_H

#include <cassert>
#include <tcutil.h>
#include <tchdb.h>

class HdbTokyoCabinetDatabase {
 public:
HdbTokyoCabinetDatabase() : hdb_(NULL), init_run_(false) {};
  ~HdbTokyoCabinetDatabase() {
    if(!tchdbclose(hdb_)){
      int errorcode = tchdbecode(hdb_);
     fprintf(stderr, "close error: %s\n", tchdberrmsg(errorcode));
    }
    tchdbdel(hdb_);
  };
  void init() {
    hdb_ = tchdbnew();
    assert(hdb_);
    bool ret = tchdbtune(hdb_, 1000000, 4 , 10, HDBTLARGE);
    if (!ret) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error: tchdbtune did not return 0: %s\n", tchdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
    init_run_=true;
  }
  void openReadOnly(std::string tokyo_cabinet_dir) {
    init();
    if(!tchdbopen(hdb_, fullFileName(tokyo_cabinet_dir).c_str(), HDBOREADER)){
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "open error: %s\n", tchdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
  }
  void create(std::string tokyo_cabinet_dir) {
    init();
    if(!tchdbopen(hdb_, fullFileName(tokyo_cabinet_dir).c_str(), HDBOWRITER | HDBOCREAT)){
      int errorcode = tchdbecode(hdb_);
      fprintf(stderr, "open error: %s\n", tchdberrmsg(errorcode));
      exit(EXIT_FAILURE);
    }
  }
  uint64_t num_entries() const {
    return tchdbrnum(hdb_);
  }
  std::string fullFileName(std::string tokyo_cabinet_dir) {
    std::string file_name = tokyo_cabinet_dir + std::string("/") + endingFilename();
    return file_name;
  }
  virtual std::string endingFilename() = 0;
 protected:
  TCHDB *hdb_;
  bool init_run_;
};

#endif
