#ifndef MAXLINK_TEXT_PARSER_H
#define MAXLINK_TEXT_PARSER_H
#include <memory>
#include <stdint.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include "num2string_gene_database.h"
#include "string2num_gene_database.h"
#include "edge_database.h"

class MaxlinkTextParser {
public:
  MaxlinkTextParser() {};
  ~MaxlinkTextParser() {};

  maxlink_types::GeneId getGeneId( maxlink_types::GeneId *gene_counter_ptr, const std::string &id_string, Num2StringGeneDatabase *num2string_gene_database, String2NumGeneDatabase *string2num_gene_database) {
    maxlink_types::GeneId gene;
    if (string2num_gene_database->translateString2Num(id_string, &gene)) {
    } else {
      maxlink_types::GeneId tmp_gene_counter = *gene_counter_ptr;
      ++tmp_gene_counter;
      *gene_counter_ptr = tmp_gene_counter;

      maxlink_types::GeneId tmp_gene1, tmp_gene2;
      string2num_gene_database->storeGene(id_string, &tmp_gene1);
      num2string_gene_database->storeGene(id_string, &tmp_gene2);
      assert(tmp_gene1 == tmp_gene2);
      assert(tmp_gene1 + 1 == *gene_counter_ptr);
      gene = tmp_gene1;
    }
    return gene;
  }

  bool loadKeyValueDatabaseFromText(Num2StringGeneDatabase *num2string_gene_database, String2NumGeneDatabase *string2num_gene_database, EdgeDatabase *edge_database, FbsDatabase *fbs_database, const char *filepath,  maxlink_types::GeneId *gene_count, float fbs_cutoff) {
    *gene_count = 0;
    maxlink_types::GeneId gene_counter = 0;
    std::string line;
    std::ifstream file(filepath);

    // Skipping the first line. It is read but not used.
    std::getline(file, line);

    int64_t line_nr = 1;

    while(std::getline(file, line)) {
      ++line_nr;
      std::string token;
      std::istringstream tokens(line);

      std::string gene1_id_str;
      std::string gene2_id_str;

      maxlink_types::Fbs fbs;
      maxlink_types::Pfc pfc;

      tokens >> gene1_id_str;
      tokens >> gene2_id_str;
      tokens >> fbs;
      tokens >> pfc;
      if (fbs <= fbs_cutoff) {
        continue;
      }
      if (gene1_id_str.size() == 0 || gene1_id_str.size() == 0) {
        std::cerr << "warning: Skipping line nr " << line_nr << " because at least of the gene identifiers was empty" << std::endl; 
        continue;
      }

      maxlink_types::GeneId gene1 = getGeneId(&gene_counter, gene1_id_str, num2string_gene_database, string2num_gene_database);
      maxlink_types::GeneId gene2 = getGeneId(&gene_counter, gene2_id_str, num2string_gene_database, string2num_gene_database);
      edge_database->storeEdge(gene1, gene2, pfc, fbs);
      fbs_database->storeFbs(fbs);

    }
    *gene_count = gene_counter;
    return true;
  }
};

#endif



