#ifndef MAXLINK_XML_PARSER_H
#define MAXLINK_XML_PARSER_H

#include <memory>
#include "maxlink.h"
#include <boost/foreach.hpp>
#include <stdint.h>
#include "config.h"

class MaxlinkXmlParser {
 public:
  MaxlinkXmlParser() {};
  ~MaxlinkXmlParser() {};
  bool loadKeyValueDatabaseFromXml(bool debug, Num2StringGeneDatabase *num2string_gene_database, String2NumGeneDatabase *string2num_gene_database, EdgeDatabase *edge_database, FbsDatabase *fbs_database, const char *xml_filename, maxlink_types::GeneId *gene_count_total, float fbs_cutoff) {
   if (debug) {
      std::cerr << "debug: entered function MaxlinkXmlParser::loadKeyValueDatabaseFromXml()" << std::endl;
   }
   int64_t edge_count = 0;
  maxlink_types::GeneId gene_count = 0;
  try {
    xml_schema::properties props;
    props.schema_location(MAXLINK_NAMESPACE, MAXLINK_SCHEMA_INSTALL_PATH);
    // Right now we load the whole xml file into memory but to make this program less memory hungry 
    // we could use the streaming parsing approach that for instance is used at
    // https://github.com/percolator/percolator/blob/master/src/Caller.cpp
    std::auto_ptr<maxlink_ns::maxlink> root(maxlink_ns::maxlink_(xml_filename, 0, props));
    BOOST_FOREACH(const maxlink_ns::gene &gene, root->gene()) {
      ++gene_count;
      if(gene_count % 10000 == 0) {
        std::cerr << gene_count << " gene id strings loaded" << std::endl;
      }
      maxlink_types::GeneId tmp_gene1, tmp_gene2;
      num2string_gene_database->storeGene(gene.gene_id(), &tmp_gene1);
      string2num_gene_database->storeGene(gene.gene_id(), &tmp_gene2);
      assert(tmp_gene1 == tmp_gene2);
      assert(tmp_gene1 + 1 == gene_count);
      if (debug) {
        std::cerr << "debug: gene id = " << gene.gene_id() << " is mapped to id nr = " << tmp_gene1 << std::endl;
      }
    }
    BOOST_FOREACH(const maxlink_ns::edge &edge, root->edge()) {
      if (edge.fbs() <= fbs_cutoff) {
        continue;
      }
      edge_count++;
      if(edge_count % 10000 == 0) {
        std::cerr << edge_count << " edges loaded" << std::endl;
      }
      maxlink_types::GeneId gene1;
      bool ret = string2num_gene_database->translateString2Num(edge.gene1(), &gene1);
      assert(ret);
      maxlink_types::GeneId gene2;
      ret = string2num_gene_database->translateString2Num(edge.gene2(), &gene2);
      assert(ret);
      if (debug) {
        std::cerr << "debug: storing edge with genes id nr = " << gene1 << " and id nr = " << gene2 << " and edge.pfc() = " << edge.pfc() << " and edge.fbs() = " << edge.fbs() << std::endl;
      }
      edge_database->storeEdge(gene1, gene2, edge.pfc(), edge.fbs());
      fbs_database->storeFbs(edge.fbs());
    }
    *gene_count_total = gene_count;
    }
  catch(const xml_schema::exception &e) {
      std::cerr << "in catch(const xml_schema::exception &e) {" << std::endl;
      std::cerr << e << std::endl;
      return false;
    }
    catch(const std::ios_base::failure &) {
      std::cerr << ": unable to open or read/write failure" << std::endl;
      return false;
    }
    return true;
  }
};

#endif



