#ifndef SHORTEST_PATH_DATABASE_H
#define SHORTEST_PATH_DATABASE_H

#include <vector>
#include <string>
#include <cassert>
#include <tcutil.h>
#include <tchdb.h>
#include "maxlink_types.h"
#include "hdb_tokyo_cabinet_database.h"

class ShortestPathDatabase : public HdbTokyoCabinetDatabase {
 public:
ShortestPathDatabase() : shortest_path_id_count_(0) {};
  ~ShortestPathDatabase() {};
  maxlink_types::ShortestPathId storeShortestPath(const maxlink_types::ShortestPath &shortest_path) {
    assert(init_run_);
    assert(hdb_);
    if (!tchdbputcat(hdb_, &shortest_path_id_count_, sizeof(shortest_path_id_count_), &shortest_path[0], shortest_path.size() * sizeof(maxlink_types::GeneId))) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error in tchdbputcat: %s\n", tchdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
    ++shortest_path_id_count_;
    return shortest_path_id_count_ -1;
  }
  maxlink_types::ShortestPath shortestPath(maxlink_types::ShortestPathId shortest_path_id) {
    assert(init_run_);
    assert(hdb_);
    int value_size = 0;
    maxlink_types::GeneId *result_ptr = static_cast<maxlink_types::GeneId *>(tchdbget(hdb_, &shortest_path_id, sizeof(shortest_path_id), &value_size));
    if (!result_ptr) {
      maxlink_types::ShortestPath empty_vec;
      return empty_vec;
    }
    assert(value_size % sizeof(maxlink_types::GeneId) == 0);
    int shortest_path_length = value_size / sizeof(maxlink_types::GeneId);
    // http://stackoverflow.com/questions/259297/how-do-you-copy-the-contents-of-an-array-to-a-stdvector-in-c-without-looping
    maxlink_types::ShortestPath shortest_path(result_ptr, result_ptr + shortest_path_length);
    free(result_ptr);
    return shortest_path;
  }
  std::string endingFilename() {
    return std::string("ShortestPathDatabase.hdb");
  };
 private:
  maxlink_types::ShortestPathId shortest_path_id_count_;
};

#endif
