//
// Some parts of this file has the origin from the file bfs-example.cpp from BOOST
// that has this Copyright notice:
//=======================================================================
// Copyright 2001 Jeremy G. Siek, Andrew Lumsdaine, Lie-Quan Lee, 
//
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//=======================================================================


#include "shortest_paths.h"
#include <iostream>
#include <vector>
#include <utility>
#include <algorithm>
#include <iostream>

#include <boost/foreach.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/breadth_first_search.hpp>
#include <boost/pending/indirect_cmp.hpp>

// Maybe, just guessing, we should analyze this link
// http://nlp.lsi.upc.edu/freeling/index.php?option=com_simpleboard&Itemid=65&func=view&id=2318&catid=5
// Dimitri got compile errors when building maxlink on a Ubuntu 12.04
// It seems it has something to do with a newer version of the boost libraries

// #include <boost/pending/integer_range.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/breadth_first_search.hpp>
#include <boost/graph/visitors.hpp>
#include <boost/property_map/property_map.hpp>
#include <boost/graph/graph_utility.hpp>

#include "edge_database.h"
#include "shortest_path_database.h"
#include "gene_shortest_path_count_database.h"
#include "genepair_shortest_path_database.h"
#include "num2string_gene_database.h"

using namespace boost;

void storeShortestPaths(bool debug, Num2StringGeneDatabase *num2string_gene_database, const EdgeDatabase &edge_database, GeneShortestPathCountDatabase *gene_shortest_path_count_database, GenePairShortestPathDatabase *endgene_shortest_path_database, ShortestPathDatabase *shortest_path_database, maxlink_types::GeneId loaded_gene_count, maxlink_types::Fbs fbs_cutoff) {
  typedef adjacency_list <vecS, vecS, undirectedS> graph_t;
  maxlink_types::GeneId numVertices = loaded_gene_count;
  typedef std::pair<maxlink_types::GeneId, maxlink_types::GeneId> E;
  std::vector<E> edge_vector;

  if (debug) {
    std::cerr << "debug: in function storeShortestPaths()" << std::endl;
    std::cerr << "debug: loaded_gene_count =" << loaded_gene_count << std::endl;
    std::cerr << "debug: fbs_cutoff =" << fbs_cutoff << std::endl;
    std::cerr << "debug: num2string_gene_database->num_entries() =" << num2string_gene_database->num_entries() << std::endl;
    std::cerr << "debug: edge_database.num_entries() =" << edge_database.num_entries() << std::endl;
    std::cerr << "debug: gene_shortest_path_count_database->num_entries() =" << gene_shortest_path_count_database->num_entries() << std::endl;
  }

  for (maxlink_types::GeneId gene_id = 0; gene_id < loaded_gene_count; ++gene_id) {
    std::vector<maxlink_types::GeneId> neighbours = edge_database.neighbours(debug, gene_id, fbs_cutoff);
    if (debug) {
      std::cerr << "debug: gene_id = " << num2string_gene_database->translateNum2String(gene_id) << std::endl;
      std::cerr << "debug: has " << neighbours.size() << " neighbours" << std::endl;
    }
    BOOST_FOREACH(maxlink_types::GeneId neighbour_gene_id, neighbours) {
      // We don't want to store every edge twice so we only add when the neighbour has a bigger id number
      if (debug) {
        std::cerr << "debug: neighbour_gene_id = " << num2string_gene_database->translateNum2String(neighbour_gene_id) << std::endl;
      }
      if (neighbour_gene_id > gene_id) {
        edge_vector.push_back(std::make_pair(gene_id, neighbour_gene_id));
      }
    }
  }

  if (debug) {
    std::cerr << "debug: edge_vector.size()" << edge_vector.size() << std::endl;
  }

  typedef graph_traits<graph_t>::vertices_size_type v_size_t;
  graph_t g(edge_vector.begin(), edge_vector.end(), v_size_t(numVertices));
  typedef graph_traits<graph_t>::vertex_descriptor Vertex;
  typedef graph_traits <graph_t>::vertices_size_type Size;
  std::vector<Vertex> p(boost::num_vertices(g));
  std::vector<Size> d(boost::num_vertices(g));
  std::fill(d.begin(), d.end(), 0);
  graph_traits<graph_t>::vertex_iterator vi2, vi_end2;
  for (tie(vi2, vi_end2) = vertices(g); vi2 != vi_end2; ++vi2) {
    p[*vi2]=*vi2;
    boost::breadth_first_search(g, *vi2, 
                                boost::visitor(
                                    boost::make_bfs_visitor(boost::record_predecessors( &p[0],   boost::on_tree_edge()))));

    graph_traits<graph_t>::vertex_iterator vi, vi_end;
    for (tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi) {

      if (*vi == *vi2 ) {
        continue;
      }
      maxlink_types::ShortestPath spath;
      Vertex iter = *vi;
      while (p[iter] != iter) {
        spath.push_back(iter);
        iter = p[iter];
      }
      if (!spath.empty()) {  
        spath.push_back(iter); 
        maxlink_types::GenePair end_genes;
        end_genes.first = *vi;
        end_genes.second = iter;
        if (end_genes.first > end_genes.second) {
           maxlink_types::ShortestPathId shortest_path_id = shortest_path_database->storeShortestPath(spath);    
           BOOST_FOREACH(maxlink_types::GeneId &gene_in_path, spath) {
            gene_shortest_path_count_database->incrementShortestPathCount(gene_in_path);
          }  
          endgene_shortest_path_database->storeShortestPath(end_genes, shortest_path_id);
        }
      }
    }
  }
}
