#ifndef NUM2STRING_GENE_DATABASE_H
#define NUM2STRING_GENE_DATABASE_H

#include <vector>
#include <string>
#include <cassert>
#include <tcutil.h>
#include <tchdb.h>
#include "maxlink_types.h"
#include "hdb_tokyo_cabinet_database.h"

class Num2StringGeneDatabase : public HdbTokyoCabinetDatabase {
 public:
Num2StringGeneDatabase() : next_gene_id_(0) {};
  ~Num2StringGeneDatabase() {};

  // 
  // Stores the gene id string together with the next available index number.
  // The output parameter gene_id is set to this index number.

  void storeGene(const std::string &id_string, maxlink_types::GeneId *gene_id) {
    assert(init_run_);
    assert(hdb_);
    // is it correct to use id_string.c_str() and id_string.size() + 1?
    // What happens with multibyte charcters in utf-8?
    if (!tchdbput(hdb_, &next_gene_id_, sizeof(next_gene_id_), id_string.c_str(), id_string.size() + 1)) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error in tchdbput: %s\n", tchdberrmsg(error_code));
      exit(EXIT_FAILURE);
    }
    *gene_id = next_gene_id_;
    ++next_gene_id_;
  }
  std::string endingFilename() {
    return std::string("Num2StringGeneDatabase.hdb");
  };
  std::string translateNum2String(maxlink_types::GeneId gene_id) {
    assert(init_run_);
    assert(hdb_);
    int value_size = 0;
    char *result_ptr = static_cast<char *>(tchdbget(hdb_, &gene_id, sizeof(gene_id), &value_size));
    if (!result_ptr) {
      std::cerr << "could not translate geneId=" << gene_id << " to an id string" << std::endl;
      assert(false);
    }
    std::string res(result_ptr);
    free(result_ptr);
    return res;
  }
private:
  maxlink_types::GeneId next_gene_id_;
};

#endif
